# mcctr.nwk.mc.file.to.newick() takes a file path of an MCcoal control file
# as an argument, and returns the species tree as a Newick format string.
# The population sizes are not put into the Newick string.
# 
# It finds the tree in the control file by looking for a line that begins with ( and ends with ;
# 
# It converts this line to an array of nodes with fields 
#   idx", "lft", "rgt", "anc", "height", "pop"
# This is then converted to Newick format.


library(stringr)
library(ape)

mcctr.nwk.outermost.comma <- function(x) {
	len <- str_length(x)
	op <- str_locate_all(x, fixed("("))[[1]][,1]
	cl <- str_locate_all(x, fixed(")"))[[1]][,1]
	br <- rep(0, len)
	br[op] <- 1
	br[cl] <- -1
	br <- cumsum(br)
	com <- str_locate_all(x, fixed(","))[[1]][,1]
	com[which(br[com] == 1)]
}


mcctr.nwk.last.colon <- function(x) {
	xx <- str_locate_all(x, fixed(":"))[[1]]
	xx[nrow(xx),1]
}

mcctr.nwk.last.close <- function(x) {
	xx <- str_locate_all(x, fixed(")"))[[1]]
	xx[nrow(xx),1]
}

mcctr.nwk.first.open <- function(x) {
	xx <- str_locate(x, fixed("("))[,1]
}


mcctr.nwk.label.pop.from.text <- function(x) {
	pop <- 0
	hash <- str_locate(x, "#")[1,]
	if (!is.na(hash[1])) {
			labeltext <- str_sub(x, 1, hash[1]-1)
		poptext <- str_sub(x, hash[2]+1, str_length(x))
		poptext <- str_trim(poptext)
		labeltext <- str_trim(labeltext)
		pop <- as.double(poptext)	
	} else {
		labeltext <- str_trim(x)
	}

	list(label=labeltext, pop=pop)
}


mcctr.nwk.height.pop.from.text <- function(x) {
	#cat(x, "\n")
	pop <- 0
	height <- 0
	hash <- str_locate(x, "#")[1,]
	heighttext <- str_sub(x, 1, hash[1]-1)
	poptext <- str_sub(x, hash[2]+1, str_length(x))
	heighttext <- str_trim(heighttext)
	poptext <- str_trim(poptext)
	height <- as.double(heighttext)
	pop <- as.double(poptext)	
	list(height=height, pop=pop)
}


mcctr.nwk.do.subtree <- function(x, n, nodes) {
	stopifnot(str_length(x) > 1)
	root <- mcctr.nwk.outermost.comma(x)
	if (length(root) == 0) {
		np <- mcctr.nwk.label.pop.from.text(x)
		nodes[n,] <- c(n, -1, -1, -1, 0, np$pop)
		rownames(nodes)[n] <- np$label
		n <- n + 1
	} else {
		#cat(x, mcctr.nwk.first.open(x), root, "\n")
		lft <- mcctr.nwk.do.subtree(str_sub(x, mcctr.nwk.first.open(x)+1, root-1), n, nodes)
		nodes <- lft$nodes
		n <- lft$n
		rgt <- mcctr.nwk.do.subtree(str_sub(x, root+1, mcctr.nwk.last.close(x)-1), n, nodes)
		nodes <- rgt$nodes
		n <- rgt$n
		rootbranch <- str_sub(x, start=mcctr.nwk.last.colon(x)+1)
		hp <- mcctr.nwk.height.pop.from.text(rootbranch)
		nodes[n,] <- c(n, lft$n-1, rgt$n-1, -1, hp$height, hp$pop)
		rownames(nodes)[n] <- paste0(rownames(nodes)[lft$n-1], rownames(nodes)[rgt$n-1])
		n <- n + 1
	}
	list(n=n, nodes=nodes)
}



mcctr.nwk.add.ancs <- function(nodes) {
	for (n in 1:nrow(nodes)) {
		lft <- nodes[n, "lft"] 
		if (lft > 0) {
			nodes[lft, "anc"] <- n
		}
		rgt <-  nodes[n, "rgt"]
		if (rgt > 0) {
			nodes[rgt, "anc"] <- n
		}
	}
	nodes
}


mcctr.nwk.node.to.newick <- function(nodes, nn) {
	if (nodes[nn, "lft"] < 0) {
		anc <- nodes[nn, "anc"]
		paste0(rownames(nodes)[nn], ":", nodes[anc, "height"])
	} else {
		lfttxt <- mcctr.nwk.node.to.newick(nodes, nodes[nn, "lft"])
		rgttxt <- mcctr.nwk.node.to.newick(nodes, nodes[nn, "rgt"])
		anc <- nodes[nn, "anc"]
		if (anc > 0) {
			paste0("(", lfttxt, ",", rgttxt, "):", nodes[anc, "height"] - nodes[nn, "height"])
		} else {
			paste0("(", lfttxt, ",", rgttxt, ");")
		}
		
	}
}


mcctr.nwk.mctext.to.newick <- function(x) {
	semicolon <- str_locate(x, ";")
	stopifnot(nrow(semicolon) == 1)
	stopifnot(!is.na(semicolon[1,1]))
	x <- str_sub(x, 1, semicolon[1,1]-1)
	nof.internal <- str_count(x, fixed(","))
	nof.nodes <- 2 * nof.internal + 1
	nodes <- matrix(0, nrow=nof.nodes, ncol=6)
	colnames(nodes) <- c("idx", "lft", "rgt", "anc", "height", "pop")
	rownames(nodes) <- rep("", nrow(nodes))
	tree <- mcctr.nwk.do.subtree(x, 1, nodes)
	nodes <- mcctr.nwk.add.ancs(tree$nodes)
	#print(nodes)
	mcctr.nwk.node.to.newick(nodes, nrow(nodes))
}


mcctr.nwk.mc.file.to.newick <- function(fpath) {
	x <- readLines(fpath)
	x <- str_trim(x)
	treeline <- grep("[(][^;]+;", x)
	stopifnot(length(treeline) == 1)
	mcctr.nwk.mctext.to.newick(x[treeline])
}


#mcctr.nwk.mc.file.to.newick("G:/DENIM-Leache10spp-Jun19/ABCD-EFGH_0p1/rep001/ctrl.txt")


#newick <- mcctr.nwk.mctext.to.newick("((((((A # 0.0143030790591871  , B # 0.0058222648685915 ): 0.0703129069092584  # 0.0080435212857954 ,C # 0.0241611597094627 ): 0.128987752637509 # 0.0248482685455267 ,D # 0.0591020718604688 ): 0.136815760322529  # 0.00461933120325266 ,(((E # 0.0088158014470808 ,F # 0.0371468747628755 ): 0.0226820780816321  # 0.0164836297699183 ,G # 0.00913737049436536 ): 0.0379333954835681  # 0.00786336426509172 ,H # 0.00750864540912296 ): 0.0617637936921209 # 0.0138246343563382 ): 0.1478357356886  # 0.00668985081404998 ,I # 0.0158182071717271 ): 0.156285424605626  # 0.00738656488534982 ,J ): 0.168798066806662  # 0.0329717542647977 ")

# x <- "((((((A # 0.0143030790591871  , B # 0.0058222648685915 ): 0.0703129069092584  # 0.0080435212857954 ,C # 0.0241611597094627 ): 0.128987752637509 # 0.0248482685455267 ,D # 0.0591020718604688 ): 0.136815760322529  # 0.00461933120325266 ,(((E # 0.0088158014470808 ,F # 0.0371468747628755 ): 0.0226820780816321  # 0.0164836297699183 ,G # 0.00913737049436536 ): 0.0379333954835681  # 0.00786336426509172 ,H # 0.00750864540912296 ): 0.0617637936921209 # 0.0138246343563382 ): 0.1478357356886  # 0.00668985081404998 ,I # 0.0158182071717271 ): 0.156285424605626  # 0.00738656488534982 ,J ): 0.168798066806662  # 0.0329717542647977 "			
# #x <- "(((A #0.021918118 , B #0.011904464 ):0.0162732  #0.012416053 , C #0.022439539 ): 0.0477004 #0.010774198 , D): 0.0503726 #0.005390638"
# 
# 
# ph <- read.tree(text=newick)
# plot(ph)


